import axios from 'axios';
import { loadProgressBar } from 'axios-progress-bar';
import 'axios-progress-bar/dist/nprogress.css';

class Api {
    constructor() {
        loadProgressBar({ showSpinner: false });

        this.axios = axios;

        this.setupInterceptors();
    }

    setupInterceptors() {
        this.setupRequestInterceptors();
        this.setupResponseInterceptors();
    }

    requestSuccessHandler(config) {
        this.resetError();
        this.waitForResponse();

        return config;
    }

    requestErrorHandler(error) {
        this.receiveResponse();
        this.setError(error);

        return Promise.reject(error);
    }

    setupRequestInterceptors() {
        const successHandler = this.requestSuccessHandler.bind(this);
        const errorHandler = this.requestErrorHandler.bind(this);

        this.axios.interceptors.request.use(successHandler, errorHandler);
    }

    responseSuccessHandler(response) {
        this.receiveResponse();

        return response;
    }

    responseErrorHandler(error) {
        this.receiveResponse();

        if (!Api.isUnauthorizedError(error)) {
            this.setError(error);

            return Promise.reject(error);
        }

        return error;
    }

    setupResponseInterceptors() {
        const successHandler = this.responseSuccessHandler.bind(this);
        const errorHandler = this.responseErrorHandler.bind(this);

        this.axios.interceptors.response.use(successHandler, errorHandler);
    }

    waitForResponse() {
        this.store.commit('fence/request/waitForResponse');
    }

    receiveResponse() {
        this.store.commit('fence/request/receiveResponse');
    }

    setError(error) {
        this.store.commit('fence/request/setError', error);
    }

    resetError() {
        this.store.commit('fence/request/resetError');
    }

    static isUnauthorizedError(error) {
        return error.response.status === 401;
    }
}

export default new Api();
