import FenceDetails from '../components/FenceDetails';
import documentTitleSuffixSetter from './documentTitleSuffixSetter';
import contentSetter from './contentSetter';

export default {
    components: {
        FenceDetails,
    },
    mixins: [
        documentTitleSuffixSetter,
        contentSetter,
    ],
    data() {
        return {
            idProperty: 'id',
            entity: {},
            entityIsLoaded: false,
            isRetrievingEntity: false,
        };
    },
    computed: {
        routeId() {
            const { $route, idProperty } = this;

            return $route.params[idProperty];
        },
        numericRouteId() {
            return Number(this.routeId);
        },
        shouldRetrieveEntityFromApi() {
            const {
                apiPath,
                routeId,
                numericRouteId,
                entity,
                isRetrievingEntity,
                idProperty,
            } = this;

            const currentId = entity[idProperty];
            const idIsLoaded = routeId === currentId
                || numericRouteId === currentId;

            return apiPath
                && !isRetrievingEntity
                && !idIsLoaded;
        },
        fullApiPath() {
            return `${this.apiPath}/${this.routeId}`;
        },
    },
    created() {
        this.loadEntityFromApi();
    },
    activated() {
        this.loadEntityFromApi();
    },
    beforeRouteUpdate(to, from, next) {
        next();

        this.loadEntityFromApi();
    },
    methods: {
        loadEntityFromApi() {
            const {
                loadEntityFromResponse: loadEntity,
                retrieveEntityFromApi: retrieveEntity,
                shouldRetrieveEntityFromApi: shouldRetrieve,
            } = this;

            if (!shouldRetrieve) return;

            const error = () => { this.entity = {}; };
            const final = () => { this.isRetrievingEntity = false; };

            retrieveEntity()
                .then(loadEntity)
                .catch(error)
                .finally(final);
        },
        retrieveEntityFromApi() {
            this.isRetrievingEntity = true;
            this.entityIsLoaded = false;

            const options = {
                method: 'GET',
                url: this.fullApiPath,
            };

            return this.$api(options);

            // return this.$api.get(this.fullApiPath, {
            //     headers: {
            //         Authorization: `Bearer ${accessToken}`,
            //     },
            // });
        },
        loadEntityFromResponse(response) {
            const { apiResponseRootKey } = this;

            this.entity = response.data[apiResponseRootKey];
            this.entityIsLoaded = true;
        },
    },
};
