import { isObject } from './typeChecks';

function fullPath(parentPath, path) {
    return `${parentPath}${path}`;
}

function getProcessedSubpaths(pathConfig, parentPath) {
    const { path, subPaths } = pathConfig;
    const newParentPath = fullPath(parentPath, path);

    // Disables linter since this function call
    // is a recursion, so it can't be after define.
    // eslint-disable-next-line no-use-before-define
    return subPaths ? processApiPaths(subPaths, newParentPath) : {};
}

function getParamOptionsObject(paramName, paramOptions) {
    return paramOptions.reduce((options, option) => {
        // Disables linter since it's a reduce accumulator.
        // eslint-disable-next-line no-param-reassign
        options[option] = {
            [paramName]: option,
        };

        return options;
    }, {});
}

function getStringObject(string, properties) {
    // Disables linter since using properties on a String
    // object is desired behavior. There are no side effects
    // since this String object is only used in API call
    // contexts, where it's safe.
    // eslint-disable-next-line no-new-wrappers
    const stringObject = new String(string);

    return Object.assign(stringObject, properties);
}

function paramProcessor(processedParams, [paramName, paramOptions]) {
    const optionsObject = getParamOptionsObject(paramName, paramOptions);
    const paramObject = getStringObject(paramName, optionsObject);

    // Disables linter since it's a reduce accumulator.
    // eslint-disable-next-line no-param-reassign
    processedParams[paramName] = paramObject;

    return processedParams;
}

function getProcessedParams(pathConfig) {
    const { params = {} } = pathConfig;
    const entries = Object.entries(params);

    return entries.reduce(paramProcessor, {});
}

function getPathProperties(pathConfig, parentPath) {
    const subPaths = getProcessedSubpaths(pathConfig, parentPath);
    const params = getProcessedParams(pathConfig);

    return {
        ...subPaths,
        params,
    };
}

function getProcessedPathString(path, parentPath) {
    return fullPath(parentPath, path);
}

function getProcessedPathObject(pathConfig, parentPath) {
    const path = fullPath(parentPath, pathConfig.path);
    const pathProperties = getPathProperties(pathConfig, parentPath);
    const pathObject = getStringObject(path, pathProperties);

    return pathObject;
}

function getProcessedPathGetter(pathConfig) {
    const pathGetter = isObject(pathConfig)
        ? getProcessedPathObject
        : getProcessedPathString;

    return pathGetter;
}

function getProcessedPath({ pathConfig, parentPath }) {
    const processedPathGetter = getProcessedPathGetter(pathConfig);
    const processedPath = processedPathGetter(pathConfig, parentPath);

    return processedPath;
}

function getPathProcessor(parentPath) {
    return (processedPaths, [pathName, pathConfig]) => {
        // Disables linter since it's a reduce accumulator.
        // eslint-disable-next-line no-param-reassign
        processedPaths[pathName] = getProcessedPath({ pathConfig, parentPath });

        return processedPaths;
    };
}

export default function processApiPaths(apiPaths, parentPath = '') {
    const entries = Object.entries(apiPaths);
    const processor = getPathProcessor(parentPath);
    const processedPaths = entries.reduce(processor, {});

    return processedPaths;
}
